/* coding: utf-8 */
/* Copyright (C) 2022 SofTek Systems, Inc. */

function BootStrapTableCellStyle(value, row, index, field){
    switch (field){
        case 'chkbox':
            return { classes: 'summary-chk-box-td' };
        case 'content':
            return { classes: 'summary-apikey-td' };
        case 'operation':
            return { classes: 'summary-apikey-action-td overflow-auto' };
        default:
            return { classes: '' };
    }

}

$(function(){

    const summaryTableId = 'host-register-apikey-summary-table';
    const summaryTableElm = $('#' + summaryTableId);
    const pageName = 'register_apikey';
  
    const sortArea = $('#sort-area');

    const enableFilterCategoryLinks = () => { 
        //カテゴリ名を押下した時のカテゴリ検索
        summaryTableFilter.enableFilterCategoryLink('[data-filteritem="category"]', 'categories');
    };

    const enableUpdateApikeyItems = () => {
        //変更モーダルを開く時
        $('#' + summaryTableId + ' [data-updateapikey="true"]').on('click', function(){
            const modal = $('#modalModifyApikey');
    
            $('#modify-apikey-prev-values').val('{}');
            $('#modify-apikey-page-info').val('{}');
    
            const id = $(this).data('id');
            $('#modify-apikey-id').val(id);
    
            const data = $(this).data('content');
            const item_names = JSON.parse($('#item-names').val());
            const update_item = $(this).data('update');
            $('#modify-apikey-target').val(update_item);
            const item = item_names[update_item];
            const title = (item.modify_name ? item.modify_name : item.name) + '変更';
            $('#modalModifyApikeyTitle').text(title);
    
            const no_target_areas = modal.find('.modify-item').not('[id="modify-' + update_item + '-area"]');
            no_target_areas.find('[name]').prop('disabled', true);
            no_target_areas.hide();
    
            const target_area = modal.find('#modify-' + update_item + '-area');
            target_area.show();
            target_area.find('[name]').prop('disabled', false);
    
            if( update_item === 'categories' ){
                modal.find('.modal-dialog').addClass('modal-lg');
            }else{
                modal.find('.modal-dialog').removeClass('modal-lg');
            }
    
            set_data(update_item, data);
            $('#exec-modify-apikey').prop('disabled', true);
            if( update_item === 'period' ){
                toggle_exec_modify_button(update_item);
            }
    
            toggle_modify_notes(update_item);
    
            $('#modalModifyApikey').modal('show');
        });
    }

    const enableUpdateApikey = () => {
        $('#' + summaryTableId + ' .exec-update-apikey-key').on('click', function(){
            //add_page_info($(this));
            const form = $(this).closest('[data-formid="form-update-apikey-key"]');
            if( $(this).attr('data-confirm') ){
                let conf = confirm($(this).attr('data-confirm'));
                if( !conf ) return;
            }
            (async () => {
                const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                    additionalAlertMessage: '更新に失敗しました。',
                    target: 'auth'
                });
                const paramsData = {
                    target: form.find('[name="form[target]"]').val(),
                    id: form.find('[name="form[id]"]').val(),
                };
                
                const updateApiKeyElm = form.find('[name="form[update_api_key]"]');
                const removeApiKeyElm = form.find('[name="form[remove_api_key]"]');
                if( updateApiKeyElm.length ) {
                    paramsData.update_api_key = updateApiKeyElm.val();
                }else if( removeApiKeyElm.length ) {
                    paramsData.remove_api_key = removeApiKeyElm.val();
                }
                dynamicSummaryTable.executeProcess({
                    paramsData,
                    path: form.attr('action'),
                    additionalAlertMessage: '更新に失敗しました。'
                });
            })();
            return false;
        });
    };

    const enableDeleteApikey = () => {
        $('#' + summaryTableId + ' [data-form="delete"]').on('submit', function(e){
            e.preventDefault();
            dynamicSummaryTable.executeProcess({
                paramsData: { id: $(this).find('[name="form[id]"]').val() },
                path: $(this).attr('action'),
                additionalAlertMessage: '削除に失敗しました。'
            });
            return false;
        });
    };

    //サマリー表の動的処理のため(bootstrap-tableのため)にbindする。でないとイベントが効かない。
    const bindSummaryTableRowEvents = () => {
         enable_bootstrap_actions('#' + summaryTableId);
         enable_dropdown_balloon(summaryTableId);
         enableUpdateApikey();
         enableDeleteApikey();
         enableUpdateApikeyItems();
         enableFilterCategoryLinks();
    }

    function set_sort_area_border(){
        //ソートタブが2段以上になった時に、余分な線が出るため調整
        const last_sort_item_offset_top = $('#sort-area .nav-link:last').offset().top;
        const tab_items = $('#sort-area .nav-link');
        const tab_items_len = tab_items.length
        tab_items.each(function(){
             elm = $(this);
             if( elm.offset().top != last_sort_item_offset_top ){
                 elm.css('border-bottom', '1px solid #ededed');
             }else{
                 elm.css('border-bottom' ,'');
             }
        });
    }

    init_sidfm();
   
    const dynamicSummaryTable = new DynamicSummaryTable({
        summaryTableId,
        bindSummaryTableRowEventsFunction: bindSummaryTableRowEvents,
        pageName,
        allFilteredIdsId: 'all-ids',
        chkboxSelector: `#${summaryTableId} .SelectApikey`,
        allChkboxSelector: `#${summaryTableId} .allApikeysSelect`,
        bulkBtnSelector: '.bulk-btn',
    });

    const summaryTableFilter = new SummaryTableFilter({dynamicSummaryTable});

    $(window).on('pageshow', function(){
         set_sort_area_border();
    });

    $('#pages_form').on('submit', function(){ set_summary_table_lines(pageName, $('#pages_per_page').val()) }); 
    activate_sidebar_item('sidebar-item-registerhostapikeyadmin');
    set_style_for_ActionArea_on_filter_dropdown_event(); 
    //adjust_balloon_dropdown_style();

    $(window).on('resize', function(){
        set_sort_area_border();
    });

    sticky_control('host-register-apikey-summary-table', pageName);
     
    //フィルタ
    summaryTableFilter.enableFilterText(
        '#filter-name',
        '#reset-filter-name',
        { fieldSelector: '#filter-name-field' },
        'name'
    ); //登録キー名検索

    summaryTableFilter.enableFilterText(
        '#filter-hostnum-limit',
        '#reset-filter-hostnum-limit',
        { fieldSelector: '#filter-hostnum-limit-field', isUnlimitCheckboxElmId: 'filter-hostnum-limit-check-unlimit' },
        'hostnum_limit'
    ); //制限ホスト数検索

    summaryTableFilter.enableFilterText(
        '#filter-hostname-prefix',
        '#reset-filter-hostname-prefix',
        { fieldSelector: '#filter-hostname-prefix-field', notregCheckboxElmId: 'filter-hostname-prefix-is-notreg' },
        'hostname_prefix'
    ); //接頭辞検索

    summaryTableFilter.enableFilterText(
        '#filter-hostname-suffix',
        '#reset-filter-hostname-suffix',
        { fieldSelector: '#filter-hostname-suffix-field', notregCheckboxElmId: 'filter-hostname-suffix-is-notreg' },
        'hostname_suffix'
    ); //接尾辞検索

    summaryTableFilter.enableFilterChangeLink('[data-name="dup_action"]'); //dup_action検索
    summaryTableFilter.enableFilterChangeLink('[data-name="is_dn_orig"]'); //is_dn_orig検索
    summaryTableFilter.enableFilterMultipleCategories('#exec-filter-categories', '#reset-filter-categories'); //カテゴリ検索
    $('#filter-buttons-area [data-filter="categories"][data-toggle="dropdown"]').parent().on('show.bs.dropdown', function(){
        const selectArea = $('#filter-summary-category-select-system-category-tab-content, #filter-summary-category-select-custom-category-tab-content'); 
        if( $('#filter-categories-is-notreg-check-box').prop('checked') ){
            selectArea.find('input').prop('disabled', true);
        }else{
            selectArea.find('input').prop('disabled', false);
        }
    });

    summaryTableFilter.enableFilterDate2('#exec-filter-date2', '#reset-filter-date2', 'period'); //有効期間検索
    summaryTableFilter.enableResetAllFilters('#filter-clear'); //全リセット

    function init_modify_custom_categories(){
        const custom_categories = JSON.parse($('#custom-categories').val());
        let cat_opts = [];
        custom_categories.forEach(function(cat){
            cat_opts.push(
                $('<option>').attr({
                    'data-toggle': 'tooltip',
                    'data-placement': 'right',
                    'data-boundary': 'window',
                    'data-title': cat.desc
                })
                .val(cat.id)
                .text(cat.name)
            )
        });
        $('#modify-apikey-selected-custom-categories').append(cat_opts);
        $('#modify-apikey-selected-custom-categories option, #modify-apikey-unselected-custom-categories option').tooltip();
    }
    init_modify_custom_categories();

    //新規登録
    $('#create-new-apikey').on('click', function(){
        $('#create-new-apikey-name-field').val('');
        $('#modalCreateNewApikey').modal('show');
    });

    $('#exec-regist-new-apikey').on('click', function(){
        if( $('#create-new-apikey-name-field').val().length === 0 ){
            alert('1文字以上入力して下さい');
            return false;
        }
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: '登録に失敗しました。',
                target: 'auth'
            });
            dynamicSummaryTable.executeProcess({
                paramsData: { name: $('#create-new-apikey-name-field').val() },
                path: $('#form-create-new-apikey').attr('action'),
                additionalAlertMessage: '登録に失敗しました。'
            });
        })();
        return false;
    });

    $('#form-create-new-apikey').on('keydown', function(e) {
        if( e.keyCode === 13 ){
            return false;
        }
    });

    //更新
    function set_modify_name_field(data){
        $('#modify-apikey-name-field').val(data);
    }

    function set_modify_hostname_prefix_field(data){
        $('#modify-apikey-hostname-prefix-field').val(data);
    }

    function set_modify_hostname_suffix_field(data){
        $('#modify-apikey-hostname-suffix-field').val(data);
    }

    function toggle_modify_hostnum_limit_detail(motion){
        const detail_area = $('#modify-hostnum-limit-detail');
        const arrow = $('#toggle-modify-hostnum-limit-detail-arrow');

        let open = () => {
            arrow.removeClass('fa-angle-down').addClass('fa-angle-up');
            detail_area.show();
        }

        let close = () => {
             arrow.removeClass('fa-angle-up').addClass('fa-angle-down');
             detail_area.hide();
        }

        if( motion === 'open' ){
            open();
        }else if( motion === 'close' ){
            close();
        }else{
            if( detail_area.is(':visible') ){
                close();
            }else{
                open();
            }
        }
    }

    function set_modify_hostnum_limit(data){
        const field = $('#modify-apikey-hostnum-limit-field');
        const check_unlimit = $('#modify-apikey-unlimit-hostnum');
        if( data > 0 ){
            field.prop('disabled', false);
            field.val(data);
            check_unlimit.prop('checked', false);
        }else{
            field.prop('disabled', true);
            field.val('');
            check_unlimit.prop('checked', true);
        }
        toggle_modify_hostnum_limit_detail('close');
    }

    function toggle_modify_dup_action_detail(motion){
        const detail_area = $('#modify-dup-action-detail');
        const arrow = $('#toggle-modify-dup-action-detail-arrow');

        let open = () => {
            arrow.removeClass('fa-angle-down').addClass('fa-angle-up');
            detail_area.show();
        }

        let close = () => {
             arrow.removeClass('fa-angle-up').addClass('fa-angle-down');
             detail_area.hide();
        }
  
        if( motion === 'open' ){
            open();
        }else if( motion === 'close' ){
            close();
        }else{
            if( detail_area.is(':visible') ){
                close();
            }else{
                open();
            }
        }
    }

    function set_modify_dup_action(data){
        $('#modify-apikey-select-dup-action').val(data);
        toggle_modify_dup_action_detail('close');
    }

    function set_modify_is_dn_orig(data){
        $('#modify-apikey-select-is-dn-orig').val(String(data));
    }

    function set_modify_period(data){
        const fields = $('#modify-apikey-period-begindate-field, #modify-apikey-period-enddate-field');
        const chkbox = $('#modify-apikey-unlimit-period');
        fields.val('');
        if( !data.begin_date && !data.end_date ){
            fields.prop('disabled', true);
            chkbox.prop('checked', true);
        }else{
            fields.prop('disabled', false);
            chkbox.prop('checked', false);
        }
        if( data.begin_date ){
            $('#modify-apikey-period-begindate-field').val(data.begin_date);
        }
        if( data.end_date ){
            $('#modify-apikey-period-enddate-field').val(data.end_date);
        }
    }

    function set_modify_categories(data){
        const system_category_id = data.system_category_id;
        const custom_category_ids = data.custom_category_ids;
        if( !system_category_id ){
            $('#modify-apikey-select-system-category option:first').prop('selected', true);
        }else{
            $('#modify-apikey-select-system-category').val(system_category_id);
        }
        $('#modify-apikey-selected-custom-category-ids').val(JSON.stringify(custom_category_ids));
        let selected_opts = [];
        let unselected_opts = [];
        $('#modify-apikey-selected-custom-categories option, #modify-apikey-unselected-custom-categories option').each(function(idx, elm){
            let value = parseInt($(elm).val());
            if( custom_category_ids.includes(value) ){
                selected_opts.push(elm);
            }else{
                unselected_opts.push(elm);
            }
        });
        selected_opts.sort(function(a, b){
            if( $(a).text() < $(b).text() ){
                return -1;
            }else{
                return 1;
            }
        });
        unselected_opts.sort(function(a, b){
            if( $(a).text() < $(b).text() ){
                return -1;
            }else{
                return 1;
            }
        });
        $('#modify-apikey-selected-custom-categories').append(selected_opts);
        $('#modify-apikey-unselected-custom-categories').append(unselected_opts);
        toggle_custom_category_warning_message();
    }

    function set_data(update_item, data){
        let prev_values = JSON.parse($('#modify-apikey-prev-values').val());
        prev_values[update_item] = data;
        $('#modify-apikey-prev-values').val(JSON.stringify(prev_values));
        if(update_item === 'name') {
            set_modify_name_field(data);
        }else if(update_item === 'hostnum_limit'){
            set_modify_hostnum_limit(data);
        }else if(update_item === 'hostname_prefix'){
            set_modify_hostname_prefix_field(data);
        }else if(update_item === 'hostname_suffix'){
            set_modify_hostname_suffix_field(data);
        }else if(update_item === 'dup_action'){
            set_modify_dup_action(data)
        }else if(update_item === 'is_dn_orig'){
            set_modify_is_dn_orig(data)
        }else if(update_item === 'period'){
            set_modify_period(data);
        }else if(update_item === 'categories'){
            set_modify_categories(data);
        }
    }

    //変更実行ボタンの切り替え
    function toggle_exec_modify_button(target){
        let is_diff = false;
        const button = $('#exec-modify-apikey');
        let prev_values = JSON.parse($('#modify-apikey-prev-values').val());
        if( target === 'name' ){
            if( prev_values.name != $('#modify-apikey-name-field').val() ){
               is_diff = true;
            }
        }else if( target === 'hostnum_limit' ){
            const num = parseInt(prev_values.hostnum_limit);
            if( $('#modify-apikey-unlimit-hostnum').prop('checked') ){
                if( num != -1 ){
                    is_diff = true;
                }
            }else{
                if( $('#modify-apikey-hostnum-limit-field').val().length > 0 && (parseInt($('#modify-apikey-hostnum-limit-field').val()) != num || num <= 0) ){
                    is_diff = true;
                }  
            }
        }else if( target === 'hostname_prefix' ){
            if( $('#modify-apikey-hostname-prefix-field').val() != prev_values.hostname_prefix ){
                is_diff = true;
            }
        }else if( target === 'hostname_suffix' ){
            if( $('#modify-apikey-hostname-suffix-field').val() != prev_values.hostname_suffix ){
                is_diff = true;
            }
        }else if( target === 'dup_action' ){
            if( $('#modify-apikey-select-dup-action').val() != prev_values.dup_action ){
                is_diff = true;
            }
        }else if( target === 'is_dn_orig' ){
            if( $('#modify-apikey-select-is-dn-orig').val() != String(prev_values.is_dn_orig) ){
                is_diff = true;
            }
        }else if( target === 'is_dn_orig' ){
            if( $('#modify-apikey-select-is-dn-orig').val() != prev_values.is_dn_orig ){
                is_diff = true;
            }
        }else if( target === 'period' ){
            if( $('#modify-apikey-unlimit-period').prop('checked') ){
                if( prev_values.period.begin_date || prev_values.period.end_date ){
                    is_diff = true;
                }
            }else{
                let begin_date = get_date($('#modify-apikey-period-begindate-field').val());
                let end_date = get_date($('#modify-apikey-period-enddate-field').val());
                if( begin_date === '' ){
                    begin_date = null;
                }
                if( end_date === '' ){
                    end_date = null;
                }
                if( begin_date != prev_values.period.begin_date || end_date != prev_values.period.end_date ){
                    is_diff = true;
                }
            }
        }else if( target === 'categories' ){
            const prev_c_category_ids = prev_values.categories.custom_category_ids;
            const c_category_ids = JSON.parse($('#modify-apikey-selected-custom-category-ids').val());
            if( c_category_ids.length === 0 ){
                is_diff = false;
            }else{
                if( prev_values.categories.system_category_id != parseInt($('#modify-apikey-select-system-category').val()) ){
                    is_diff = true;
                }
                if( !is_diff ){
                    if( prev_c_category_ids.length != c_category_ids.length ){
                        is_diff = true;
                    }
                    if( !is_diff ){
                        for( let i = 0; i < c_category_ids.length; i++){
                            if( !prev_c_category_ids.includes(parseInt(c_category_ids[i])) ){
                                is_diff = true;
                                break;
                            }
                        }
                    }
                }
            }
        }

        if( is_diff ){
            button.prop('disabled', false);
        }else{
            button.prop('disabled', true);
        }
    }

    $(
        [
            '#modify-apikey-name-field',
            '#modify-apikey-hostname-prefix-field',
            '#modify-apikey-hostname-suffix-field',
            '#modify-apikey-hostnum-limit-field',
        ].join(', ')
    ).on('input', function(){
        toggle_exec_modify_button($('#modify-apikey-target').val());
    });

    $('#modify-apikey-select-dup-action, #modify-apikey-select-is-dn-orig').on('change', function(){
        toggle_exec_modify_button($('#modify-apikey-target').val());
    });

    $('#modify-apikey-period-begindate-field, #modify-apikey-period-enddate-field').on('change', function(){
        toggle_exec_modify_button($('#modify-apikey-target').val());
    });


    function toggle_modify_notes(target){
        let notes_area = $('#modify-apikey-notes');
        if( [ 'hostname_prefix', 'hostname_suffix', 'dup_action' ].includes(target) ){
            notes_area.show();
            let dup_action_part = $('#modify-apikey-notes-dup-action-part');
            let no_dup_action_part = $('#modify-apikey-notes-no-dup-action-part');
            if( target === 'dup_action'){
                dup_action_part.show();
                no_dup_action_part.hide();
            }else{
                dup_action_part.hide();
                no_dup_action_part.show();
            }
        }else{
            notes_area.hide();
        }
    }

    function toggle_custom_category_warning_message(){
        const selected_ccategory_ids = JSON.parse($('#modify-apikey-selected-custom-category-ids').val());
        const message_elm = $('#warning-no-category');
        if( selected_ccategory_ids.length === 0 ){
            message_elm.show();
        }else{
            message_elm.hide();
        }
    }

    //カテゴリの選択時
    $('#modify-apikey-select-custom-categories-button').on('click', function(){
        move_selected_items(
            '#modify-apikey-selected-custom-category-ids',
            '#modify-apikey-unselected-custom-categories',
            '#modify-apikey-selected-custom-categories',
            false
        );
        toggle_exec_modify_button($('#modify-apikey-target').val());
        toggle_custom_category_warning_message();
        $('#modify-apikey-selected-custom-categories option').tooltip();
    });

    $('#modify-apikey-unselect-custom-categories-button').on('click', function(){
        move_selected_items(
            '#modify-apikey-selected-custom-category-ids',
            '#modify-apikey-selected-custom-categories',
            '#modify-apikey-unselected-custom-categories', 
            true
        );
        toggle_exec_modify_button($('#modify-apikey-target').val());
        toggle_custom_category_warning_message();
        $('#modify-apikey-unselected-custom-categories option').tooltip();
    });

    $('#modify-apikey-select-system-category').on('change', function(){
        toggle_exec_modify_button($('#modify-apikey-target').val());
    });

    //制限ホスト数の制限・無制限の切り替え
    $('#modify-apikey-unlimit-hostnum').on('change', function(){
        const field = $('#modify-apikey-hostnum-limit-field');
        if( $(this).prop('checked') ){
            field.prop('disabled', true);
        }else{
            field.prop('disabled', false);
        }
        toggle_exec_modify_button($('#modify-apikey-target').val());
    });

    //有効期限の制限・無期限の切り替え
    $('#modify-apikey-unlimit-period').on('change', function(){
        const fields = $('#modify-apikey-period-begindate-field, #modify-apikey-period-enddate-field');
        if( $(this).prop('checked') ){
            fields.prop('disabled', true);
        }else{
            fields.prop('disabled', false);
        }
        toggle_exec_modify_button($('#modify-apikey-target').val());
    });

    //制限ホスト数の説明の切り替え
    $('#toggle-modify-hostnum-limit-detail').on('click', function(){
        toggle_modify_hostnum_limit_detail('toggle');
    });

    //dup_actionの説明の切り替え
    $('#toggle-modify-dup-action-detail').on('click', function(){
        toggle_modify_dup_action_detail('toggle');
    });

    //変更form送信時
    function add_page_info(form){
        const page_info = {
            'offset': $('#pages_offset').val(),
            'per_page': $('#pages_per_page').val(),
            'order': $('#pages_order').val(),
            'filters': JSON.parse($('#pages_filters').val()),
            'checked': JSON.parse($('#pages_checked').val()),
            'all_checked': $('#pages_all_checked').val(),
        };
        form.find('[name="form[page_info]"]').val(JSON.stringify(page_info));
    }

    function validate_modify_apikey_value(target){
        let err_message = '';
        if( target === 'name' ){
            if( $('#modify-apikey-name-field').val().length === 0 ){
                err_message = '1文字以上入力して下さい';
            }
        }else if( target === 'hostnum_limit' ){
            if( !$('#modify-apikey-unlimit-hostnum').prop('checked') ){
                if( parseInt($('#modify-apikey-hostnum-limit-field').val()) <= 0 ){
                    err_message = '制限をかける場合は、1以上の数字を入力して下さい';
                }
            }
        }else if( target === 'hostname_prefix'){
            let prefix = $('#modify-apikey-hostname-prefix-field').val();
            if( prefix.length > 0 && !prefix.match(/^[0-9a-zA-Z._-]+$/) ){
                err_message = '接頭辞には、半角英数字及び「.(ピリオド)」「_(アンダースコア)」「-(ハイフン)」のいずれかのみ設定可能です';
            }
        }else if( target === 'hostname_suffix'){
            let suffix = $('#modify-apikey-hostname-suffix-field').val();
            if( suffix.length > 0 && !suffix.match(/^[0-9a-zA-Z._-]+$/) ){
                err_message = '接尾辞には、半角英数字及び「.(ピリオド)」「_(アンダースコア)」「-(ハイフン)」のいずれかのみ設定可能です';
            }
        }else if( target === 'period' ){
            let begin_date = get_date($('#modify-apikey-period-begindate-field').val());
            let end_date = get_date($('#modify-apikey-period-enddate-field').val());
            if( begin_date === false || end_date === false ){
                err_message = '日付は、西暦4桁、及び月と日のある日付を入力して下さい。\n※数字は半角で入力して下さい。';
            }else if( $('#modify-apikey-period-begindate-field').val().length === 0 && $('#modify-apikey-period-enddate-field').val().length === 0 ){
                err_message = '有効期限が無期限でない場合は、日付を入力して下さい';
            }else if( begin_date && end_date ){
                begin_date = new Date(begin_date);
                end_date = new Date(end_date);
                if( begin_date > end_date ){
                    err_message = '開始日は、終了日以前の日付にして下さい';
                }
            } 
        }
        return err_message;
    }

    //変更の実行
    $('#exec-modify-apikey').on('click', function(){
        $('#form-modify-apikey').submit();
    });

    $('#form-modify-apikey').on('submit', function(e) {
        e.preventDefault();
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: '更新に失敗しました。',
                target: 'auth'
            });
            const target = $('#modify-apikey-target').val();
            const err_message = validate_modify_apikey_value(target);
            if( err_message.length > 0 ){
                alert(err_message);
                return false;
            }

            const form = $(this);
            const paramsData = {};
            form.find('[name][type="hidden"]').each((idx, elm) => {
                elm = $(elm);
                if( !elm.prop('disabled') ){
                    switch (elm.attr('name')) {
                        case 'form[target]':
                            paramsData.target = elm.val();
                            break;
                        case 'form[id]':
                            paramsData.id = elm.val();
                            break;
                    }
                }
            });

            form.find('[name]:visible').each((idx, elm) => {
                elm = $(elm);
                switch (elm.attr('name')) {
                    case 'form[name]':
                        paramsData.name = elm.val();
                        break;
                    case 'form[is_unlimit_hostnum]':
                        if( elm.prop('checked') ) paramsData.is_unlimit_hostnum = elm.val();
                        break;
                    case 'form[hostnum_limit]':
                        if( !elm.prop('disabled') ) paramsData.hostnum_limit = elm.val();
                        break;
                    case 'form[hostname_prefix]':
                        paramsData.hostname_prefix = elm.val();
                        break;
                    case 'form[hostname_suffix]':
                        paramsData.hostname_suffix = elm.val();
                        break;
                    case 'form[dup_action]':
                        paramsData.dup_action = elm.val();
                        break;
                    case 'form[is_dn_orig]':
                        paramsData.is_dn_orig = elm.val();
                        break;
                    case 'form[system_category_id]':
                        paramsData.system_category_id = elm.val();
                        paramsData.custom_category_ids = form.find('[name="form[custom_category_ids]"]').val();
                        break;
                    case 'form[is_unlimit_period]':
                        if( elm.prop('checked') ) paramsData.is_unlimit_period = elm.val();
                        break;
                    case 'form[begin_date]':
                        if( !elm.prop('disabled') ) paramsData.begin_date = elm.val();
                        break;
                    case 'form[end_date]':
                        if( !elm.prop('disabled') ) paramsData.end_date = elm.val();
                        break;
                }
            });
            dynamicSummaryTable.executeProcess({
                paramsData,
                path: $('#form-modify-apikey').attr('action'),
                additionalAlertMessage: '更新に失敗しました。'
            });
            return false;
            //$('#form-modify-apikey').submit();
        })();
        return false;
    });

    //クリップボードにコピー
    const clipboard = new ClipboardJS('.copy-apikey');

    //クリップボードがサポートされていなければアラートを表示 
    $(document).on('click', '.copy-apikry', function(e){
        e.stopPropagation();
        $('#tooltip-copy-btn').tooltip('hide');
        if(clipboard_supported === false) {
            alert('このブラウザではコピー機能はサポートされておりません');
            return false;
        }
    });

    clipboard.on('success', function(e){
        let trigger = $(e.trigger);
        trigger.tooltip('show');
        setTimeout(function (){
            trigger.tooltip('hide');
            e.clearSelection();
        }, 1000);
    });

    clipboard.on('error', function(e) {
        alert('コピーに失敗しました');
        e.clearSelection();
    });

    //一括操作
    function create_bulk_tr_elm(name){
        let tr = $('<tr>');
        tr.append(
            $('<td>').attr('class', 'text-left text-break').text(name)
        );
        return tr;
    }

    //一括削除
    $('#bulk-delete-button').on('click', function(){
        let checked = JSON.parse($('#pages_checked').val());
        $('#bulk-delete-ids').val(JSON.stringify(checked));
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: '削除に失敗しました。'
            });
            if( roleOrCategoryError ) return;
            $.post( $('#apikey-names-path').val(), { 'ids': JSON.stringify(checked) }, function(names){
                $('#modalBulkDeleteTbody *').remove();
                const slice_num = 20;
                let name_elms = [];
                const target = $('#modalBulkDeleteTbody');
                names.slice(0, 20).forEach( name => name_elms.push(create_bulk_tr_elm(name)) );
                target.append(name_elms);
                $('#modalBulkDelete').modal('show');
                setTimeout(function(){
                    names.slice(slice_num).forEach( name => target.append(create_bulk_tr_elm(name)) );
                });
            });
        })();
    });

    $('#exec-bulk-delete').on('click', function(){
        let conf = confirm('一括削除します。よろしいですか?');
        if( !conf ) return;
        dynamicSummaryTable.executeProcessWithProgressBar({
            ids: JSON.parse($('#bulk-delete-ids').val()),
            paramsData: {},
            idsParamsName: 'ids',
            path: $('#form-bulk-delete').attr('action'),
        });
        return false; 
        //$('#form-bulk-delete').submit();
    });
    
});
