#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmUsersControllerSupport
  # 既存のメソッドとメソッド名が被らないように注意

  # ソート
  # orderはascかdescのみを指定する
  def sort_users_content(content:, order:, sidfm_roles:)
    last_index = order.rindex('_')
    column = order.slice(0..last_index - 1)
    order = order.slice(last_index + 1..order.size - 1)

    case column
    when 'login' then
      content = content.order(%|LOWER(login COLLATE "C") #{order.upcase}|)
    when 'lastname' then
      content = content.order(%|LOWER(lastname COLLATE "C") #{order.upcase}|)
                       .order(%|LOWER(login COLLATE "C") #{order.upcase}|)
    when 'mail_notification' then
      content = content.order(%|
        CASE 
        WHEN users.mail_notification = 'only_my_events' THEN
          #{order == 'asc' ? '0' : '10'}
        ELSE 
          #{order == 'asc' ? '10' : '0'}
        END
      |).order('LOWER(login COLLATE "C") ASC')
    when 'lock' then
      if sidfm_roles.include?(:admin) then
        content = content.order(%|
          CASE 
          WHEN status = #{User::STATUS_LOCKED} THEN
            #{order == 'asc' ? '0' : '10'}
          ELSE
            #{order == 'asc' ? '10' : '0'}
          END
        |).order(%|LOWER(login COLLATE "C") #{order.upcase}|)
      end
    when 'last_login_on' then
      content = content.order(%|
        last_login_on #{order.downcase == 'asc' ? 'DESC NULLS LAST' : 'ASC NULLS FIRST'}
      |)
    else
      content = content.order(column => order.to_sym).order(%|LOWER(login COLLATE "C") #{order.upcase}|)
    end
    content
  end

  # フィルタ適用
  def filter_users_content(content:, filters:, sidfm_project_id:, current_user:, sidfm_roles:)

    if filters[:login] && filters[:login].length > 0 then
      if filters[:login] != current_user.login then
        content = content.where('login ILIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, filters[:login])}%")
      else
        content = content.where(:login => filters[:login])
      end
    end

    if filters[:name] && filters[:name].length > 0 then
      content = content.where(
        'firstname ILIKE ? OR lastname ILIKE ?',
        "%#{ActiveRecord::Base.send(:sanitize_sql_like, filters[:name])}%",
        "%#{ActiveRecord::Base.send(:sanitize_sql_like, filters[:name])}%"
      )
    end

    if filters[:auth] && filters[:auth].length > 0 then
      auth = filters[:auth].to_i
      if auth == -1 then
        content = content.where(auth_source_id: nil)
      else
        content = content.joins("LEFT OUTER JOIN auth_sources ON users.auth_source_id = auth_sources.id")
                         .where("auth_sources.id = ?", auth)
      end
    end

    if filters[:role] && filters[:role].length > 0 then
      role_id = filters[:role].to_i
      #if auth == -1 then
      #  content = content.where.not(:admin => true).where.not(:id => Member.all.select(:user_id))
      #else
        content = content.where(
          :id => content.joins("
            LEFT OUTER JOIN members ON project_id = #{sidfm_project_id} AND members.user_id=users.id
          ")
          .joins("LEFT OUTER JOIN member_roles ON members.id = member_roles.member_id")
          .joins("LEFT OUTER JOIN roles ON member_roles.role_id = roles.id")
          .where("roles.id = ?", role_id)
          .select(:id)
        )
      #end
    end

    if filters[:category] && filters[:category].to_i != 0 then
      category = filters[:category].to_i
      if !sidfm_roles.include?(:admin) && !current_user.categories.where(:id => category).exists? then
        category = 0
      end
      content = content.where(
        :id => content.joins("
          LEFT OUTER JOIN sidfmd_categories_users ON sidfmd_categories_users.user_id = users.id
        ")
        .joins("
          LEFT OUTER JOIN sidfmd_categories ON sidfmd_categories_users.sidfmd_category_id = sidfmd_categories.id"
        )
        .where("sidfmd_categories.id = ?", category)
        .select(:id)
      )
    end

    if sidfm_roles.include?(:user_admin) &&
      filters[:group] &&
      filters[:group].length > 0 &&
      filters[:group].to_i != 0
    then
      if filters[:group].to_i == -1 then
        content = content.where.not(
          :id => SidfmdCatGroupsUser.select(:user_id)
        )
      else
        content = content.where(
          :id => SidfmdCatGroupsUser.where(:sidfmd_cat_group_id => filters[:group].to_i).select(:user_id)
        )
      end
    end

    if !filters[:mail_notification].nil? && filters[:mail_notification].length > 0 then
      if filters[:mail_notification].to_s == "true" then
        content = content.where(:mail_notification => "only_my_events")
      elsif filters[:mail_notification].to_s == "false" then
        content = content.where(:mail_notification => "none")
      end
    end

    if filters[:last_login_on] && filters[:last_login_on].size > 0 then
      begin_date = filters[:last_login_on][0].to_s
      end_date = filters[:last_login_on][1].to_s
      begin
        if begin_date == 'not-login-users' then
          content = content.where('last_login_on IS NULL')
        elsif begin_date.size > 0 && end_date.size > 0 then
          begin_date = Date.parse(begin_date)
          end_date = Date.parse(end_date)
          content = content.where('last_login_on >= ? AND last_login_on < ?', begin_date, end_date + 1.days)
        elsif begin_date.size > 0 then
          begin_date = Date.parse(begin_date)
          content = content.where('last_login_on >= ?', begin_date)
        elsif  end_date.size > 0 then
          end_date = Date.parse(end_date)
          content = content.where('last_login_on < ?', end_date + 1.days)
        end
      rescue ArgumentError => e
        log_exception(e)
        redirect_to :action => :index, :notice => '検索した日付に誤りがあります。' and return
      end
    end

    if sidfm_roles.include?(:admin) && filters[:locked] && filters[:locked].to_s.size > 0 && sidfm_roles.include?(:admin) then
      if filters[:locked].to_s == 'true' then
        content = content.where(:status => User::STATUS_LOCKED)
      elsif filters[:locked].to_s == 'false' then
        content = content.where.not(:status => User::STATUS_LOCKED)
      end
    end

    content
  end

end
